@@ -1,7 +1,6 @@
 // staart is a Rust implementation of a tail-like program for Linux
 // Copyright 2020-2021 Anthony Martinez
-//
-// Licensed under the Apache License, Version 2.0, <LICENSE-APACHE or
+// // Licensed under the Apache License, Version 2.0, <LICENSE-APACHE or
 // http://apache.org/licenses/LICENSE-2.0> or the MIT license <LICENSE-MIT or
 // http://opensource.org/licenses/MIT>, at your option. This file may not be
 // copied, modified, or distributed except according to those terms.
@@ -17,9 +16,9 @@
 //! ```no_run
 //! use std::thread::sleep;
 //! use std::time::Duration;
-//! use staart::TailedFile;
+//! use staart::{StaartError, TailedFile};
 //!
-//! fn main() -> Result<(), Box<dyn std::error::Error>> {
+//! fn main() -> Result<(), StaartError> {
 //!     let delay = Duration::from_millis(100);
 //!     let args: Vec<String> = std::env::args().collect();
 //!     let path = &args[1].as_str();
@@ -36,6 +35,12 @@ use std::io::{BufReader, Read, Seek, SeekFrom};
 use std::os::linux::fs::MetadataExt;
 use std::path::Path;
 
+mod errors;
+
+pub use errors::StaartError;
+
+type Result<T> = std::result::Result<T, StaartError>;
+
 /// [`TailedFile`] tracks the state of a file being followed. It offers
 /// methods for updating this state, and printing data to `stdout`.
 pub struct TailedFile<T> {
@@ -55,7 +60,7 @@ impl<T: AsRef<Path> + Copy> TailedFile<T> {
     /// # Propagates Errors
     /// - If the path provided does not exist, or is not readable by the current user
     /// - If file metadata can not be read
-    pub fn new(path: T) -> std::io::Result<TailedFile<T>> {
+    pub fn new(path: T) -> Result<TailedFile<T>> {
         let inode: u64;
         let pos: u64;
 
@@ -71,7 +76,7 @@ impl<T: AsRef<Path> + Copy> TailedFile<T> {
 
     /// Reads new data for an instance of `staart::TailedFile` and returns
     /// `Result<[u8; 65536]>`
-    pub fn read(&mut self, file: &File) -> Result<[u8; 65536], Box<dyn std::error::Error>> {
+    pub fn read(&mut self, file: &File) -> Result<[u8; 65536]> {
         let mut reader = BufReader::with_capacity(65536, file);
         let mut data: [u8; 65536] = [0u8; 65536];
 
@@ -83,42 +88,36 @@ impl<T: AsRef<Path> + Copy> TailedFile<T> {
     }
 
     /// Prints new data read on an instance of `staart::TailedFile` to `stdout`
-    pub fn follow(&mut self) -> Result<(), Box<dyn std::error::Error>> {
+    pub fn follow(&mut self) -> Result<()> {
         let fd = File::open(self.path)?;
         let meta = fd.metadata()?;
-        self.check_rotate(&meta)?;
-        self.check_truncate(&meta)?;
+        self.check_rotate(&meta);
+        self.check_truncate(&meta);
         let data = self.read(&fd)?;
 
-        if let Ok(printable) = std::str::from_utf8(&data) {
-            print!("{}", printable)
-        } else {
-            println!("Encountered non-UTF8 bytes.")
-        }
+        let printable = std::str::from_utf8(&data)?;
+	print!("{}", printable);
 
         Ok(())
     }
 
     /// Checks for file rotation by inode comparision in Linux-like systems
-    fn check_rotate(&mut self, meta: &Metadata) -> Result<(), Box<dyn std::error::Error>> {
+    fn check_rotate(&mut self, meta: &Metadata) {
         let inode = meta.st_ino();
         if inode != self.inode {
             self.pos = 0;
             self.inode = inode;
         }
 
-        Ok(())
     }
 
     /// Checks for file truncation by length comparision to the previous read position
-    fn check_truncate(&mut self, meta: &Metadata) -> Result<(), Box<dyn std::error::Error>> {
+    fn check_truncate(&mut self, meta: &Metadata) {
         let inode = meta.st_ino();
         let len = meta.len();
         if inode == self.inode && len < self.pos {
             self.pos = 0;
         }
-
-        Ok(())
     }
 }
 
@@ -173,7 +172,7 @@ mod tests {
         let mut f = File::create(&path).unwrap();
         f.write_all(more_test_data).unwrap();
 
-        assert!(tailed_file.check_rotate(&f.metadata().unwrap()).is_ok());
+        tailed_file.check_rotate(&f.metadata().unwrap());
         assert_eq!(tailed_file.inode, f.metadata().unwrap().st_ino())
     }
 
@@ -192,7 +191,7 @@ mod tests {
         let mut f = File::create(&path).unwrap();
         f.write_all(more_test_data).unwrap();
 
-        assert!(tailed_file.check_truncate(&f.metadata().unwrap()).is_ok());
+        tailed_file.check_truncate(&f.metadata().unwrap());
         assert_eq!(tailed_file.pos, 0)
     }
 }
